#!/usr/bin/python

"""
Class defining the Rosenbrock function
"""

import numpy

class Rosenbrock:
  """
  The Rosenbrock function
  """
  def __init__(self, dimension):
    """
    Constructor
    """
    self.dimension = dimension

  def __call__(self, x):
    """
    Get the value of the Rosenbrock function at a specific point
    """
    return numpy.sum(100.0 * (x[1:] - x[:-1]**2.0)**2.0 + (1. - x[:-1])**2.0)

  def gradient(self, x):
    """
    Evaluates the gradient of the function
    """
    xm = x[1:-1]
    xm_m1 = x[:-2]
    xm_p1 = x[2:]
    der = numpy.zeros(x.shape, x.dtype)
    der[1:-1] = 200. * (xm - xm_m1**2.) - 400. * (xm_p1 - xm**2.) * xm - 2. * (1. - xm)
    der[0] = -400. * x[0] * (x[1] - x[0]**2.) - 2. * (1. - x[0])
    der[-1] = 200. * (x[-1] - x[-2]**2.)
    return der

  def hessian(self, x):
    """
    Evaluates the gradient of the function
    """
    H = numpy.diag(-400. * x[:-1], 1) - numpy.diag(400. * x[:-1],-1)
    diagonal = numpy.zeros(len(x), x.dtype)
    diagonal[0] = 1200. * x[0]**2. - 400. * x[1] + 2.
    diagonal[-1] = 200.
    diagonal[1:-1] = 202. + 1200. * x[1:-1]**2. - 400. * x[2:]
    H += numpy.diag(diagonal)
    return H

if __name__ == '__main__':
  import criterion
  import step
  import optimizer
  import line_search

  import AppendList

  startPoint = numpy.empty(2, numpy.float)
  startPoint[0] = -1.01
  startPoint[-1] = 1.01

  print "Standard optimization with a gradient step, monotony criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.GradientStep(), criterion = criterion.MonotonyCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.SimpleLineSearch(stepSize = 0.001))
  print optimi.optimize()
  print "Standard optimization with a gradient step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.SimpleLineSearch(stepSize = 0.001))
  print optimi.optimize()
  print "Standard optimization with a gradient step, absolute value criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.GradientStep(), criterion = criterion.AbsoluteValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.SimpleLineSearch(stepSize = 0.001))
  print optimi.optimize()

  print "Standard optimization with a Newton step, monotony criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.NewtonStep(), criterion = criterion.MonotonyCriterion(0.00001, 100), x0 = startPoint, line_search = line_search.SimpleLineSearch())
  print optimi.optimize()
  print "Standard optimization with a Newton step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.NewtonStep(), criterion = criterion.RelativeValueCriterion(0.00001, 100), x0 = startPoint, line_search = line_search.SimpleLineSearch())
  print optimi.optimize()

  pointList = AppendList.AppendList()
  print "Standard optimization with a Newton step, absolute value criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.NewtonStep(), criterion = criterion.AbsoluteValueCriterion(0.00001, 100), x0 = startPoint, record = pointList, line_search = line_search.SimpleLineSearch())
  print optimi.optimize()
  print "Set of points used in the last optimization"
  print pointList.points

  print "Damped optimization with a gradient step, monotony criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.GradientStep(), criterion = criterion.MonotonyCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.DampedLineSearch(dampedError = 0.001, minStepSize = 0.0001))
  print optimi.optimize()
  print "Damped optimization with a gradient step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.DampedLineSearch(dampedError = 0.001, minStepSize = 0.0001))
  print optimi.optimize()
  print "Damped optimization with a gradient step, absolute value criterion"
  optimi = optimizer.StandardOptimizer(function = Rosenbrock(2), step_kind = step.GradientStep(), criterion = criterion.AbsoluteValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.DampedLineSearch(dampedError = 0.001, minStepSize = 0.0001))
  print optimi.optimize()
