"""
A list of standard convergence criteria based on the number of iterations, the last values taken by the cost function and the associated points
"""

import math

class MonotonyCriterion(object):
  """
  A simple criterion that stops when the values of the function starts to rise again
  """
  def __init__(self, error, iterationMax):
    """
    Initializes the criterion with a max number of iterations and an error fraction for the monotony test
    """
    self.error = error
    self.iterationMax = iterationMax

  def __call__(self, iteration, old_value, new_value, **kwargs):
    """
    Computes the stopping criterion
    """
    return ((iteration > self.iterationMax) or ((new_value > old_value) * (1. + self.error)).all())


class RelativeValueCriterion(object):
  """
  The relative criterion stops the optimization when the relative error of the value is below a certain level
  """
  def __init__(self, error, iterationMax):
    """
    Initializes the criterion with a max number of iterations and an error fraction
    """
    self.error = error
    self.iterationMax = iterationMax

  def __call__(self, iteration, old_value, new_value, **kwargs):
    """
    Computes the stopping criterion
    """
    return ((iteration > self.iterationMax) or (abs(new_value - old_value) / (new_value + old_value + 10e-127) < self.error))

class AbsoluteValueCriterion(object):
  """
  The relative criterion stops the optimization when the absolute error of the value is below a certain level
  """
  def __init__(self, error, iterationMax):
    """
    Initializes the criterion with a max number of iterations and an error fraction
    """
    self.error = error
    self.iterationMax = iterationMax

  def __call__(self, iteration, old_value, new_value, **kwargs):
    """
    Computes the stopping criterion
    """
    return ((iteration > self.iterationMax) or (abs(new_value - old_value) < self.error))

