
"""
A damped line search
"""

class DampedLineSearch(object):
  """
  A damped line search, takes a point and a direction.
  Tests a new point for minimization, if it is greater that the current cost times (1 + error),
    the step is divided by two, until the step is too small
  """
  def __init__(self, **kwargs):
    """
    Needs to have :
      - a minimum step size (minStepSize)
      - a factor to allow the cost to rise a little bit (dampedError)
    Can have :
      - a step modifier, a factor to modulate the step (stepSize = 1.)
    """
    self.minStepSize = kwargs.get('minStepSize', 1.)
    self.dampedError = kwargs.get('dampedError', 1.)
    self.stepSize = kwargs.get('stepSize', 1.)

  def __call__(self, origin, step, function, state, **kwargs):
    """
    Returns a good candidate
    Parameters :
      - origin is the origin of the search
      - step is the step that will be used
      - function is the function to minimize
      - state is the state of the optimizer
    """
    stepSize = self.stepSize
    currentValue = function(origin)
    optimalPoint = origin + stepSize * step
    newValue = function(optimalPoint)

    while(newValue > currentValue * (1. + self.dampedError)):
      stepSize /= 2.
      if stepSize < self.minStepSize:
        break
      optimalPoint = origin + stepSize * step
      newValue = function(optimalPoint)
    else:
      state['step_size'] = stepSize
      return optimalPoint
    state['step_size'] = 0.
    return origin

