
"""
Line Search with the quadratic interpolation method with the computation of the gradient of the function
"""

import math

class QuadraticGradientInterpolationSearch(object):
  """
  Line Search with the quadratic interpolation when the gradient of the function is provided
  """
  def __init__(self, **kwargs):
    """
    Needs to have :
      - a minimum step size (minStepSize)
    Can have :
      - a step modifier, a factor to modulate the step (stepSize = 1.)
    """
    self.minStepSize = kwargs.get('minStepSize', 1.)
    self.stepSize = kwargs.get('stepSize', 1.)
    self.goldenNumber = (math.sqrt(5) + 1) / 2.

  def __call__(self, origin, step, function, state, **kwargs):
    """
    Returns a good candidate
    Parameters :
      - origin is the origin of the search
      - step is the step that will be used
      - function is the function to minimize
      - state is the state of the optimizer
    """
    ak = 0
    v_ak = function(origin + ak * step)
    bk = self.stepSize
    v_bk = function(origin + bk * step)

    uk = ak + (1 - self.goldenNumber) * (bk - ak)
    v_uk = function(origin + uk * step)
    lk = ak + self.goldenNumber * (bk - ak)
    v_lk = function(origin + lk * step)

    while True:
      if v_uk < v_lk:
        if bk - lk < self.minStepSize:
          state['step_size'] = uk
          return origin + uk * step
        ak = lk
        v_ak = v_lk
        lk = uk
        v_lk = v_uk
        uk = ak + self.goldenNumber * (bk - ak)
        v_uk = function(origin + uk * step)
      else:
        if uk - ak < self.minStepSize:
          state['step_size'] = lk
          return origin + lk * step, {'stepSize' : lk}
        bk = uk
        v_bk = v_uk
        uk = lk
        v_uk = v_lk
        lk = ak + (1 - self.goldenNumber) * (bk - ak)
        v_lk = function(origin + lk * step)
